package com.scbfkj.uni.library;

import org.apache.commons.pool2.BaseKeyedPooledObjectFactory;
import org.apache.commons.pool2.KeyedObjectPool;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.commons.pool2.impl.GenericKeyedObjectPool;
import org.apache.commons.pool2.impl.GenericKeyedObjectPoolConfig;
import org.graalvm.polyglot.Context;
import org.graalvm.polyglot.Source;
import org.graalvm.polyglot.Value;

public final class ScriptEngineUtil {

    private final static String TO_STRING_SCRIPT = """
            (value) => {
                if (typeof value === 'string') return value
                return JSON.stringify(value);
            }
            """;


    private final static KeyedObjectPool<String, Value> scriptPool = new GenericKeyedObjectPool<>(new BaseKeyedPooledObjectFactory<String, Value>() {
        @Override
        public Value create(String script) throws Exception {
            return Context.create("js").eval(Source.create("js", script));
        }

        @Override
        public PooledObject<Value> wrap(Value value) {
            return new DefaultPooledObject<>(value);
        }

        @Override
        public void destroyObject(String key, PooledObject<Value> p) throws Exception {
            super.destroyObject(key, p);
        }
    }, new GenericKeyedObjectPoolConfig<>());


    public static String eval(String script, Object... args) throws Exception {
        Value function = scriptPool.borrowObject(script);
        try {
            if (function.canExecute()) {
                Value result = function.execute(args);
                return toString(result);
            } else {
                return toString(function);
            }
        } finally {
            scriptPool.returnObject(script, function);
        }
    }

    private static String toString(Value value) throws Exception {
        Value function = scriptPool.borrowObject(TO_STRING_SCRIPT);
        try {
            return function.execute(value).asString();
        } finally {
            scriptPool.returnObject(TO_STRING_SCRIPT, function);
        }
    }
}
