package com.scbfkj.uni.service;

import com.scbfkj.uni.library.DataAliasGetUtil;
import com.scbfkj.uni.library.ScriptEngineUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.process.DataBase;
import jakarta.el.MethodNotFoundException;
import org.springframework.stereotype.Service;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class DataProcessService {
    public Map<String, Object> process(Map<String, Object> inData) throws Exception {
        Optional<String> serviceIdOpt = DataAliasGetUtil.getValue("serviceid", inData);
        if (serviceIdOpt.isEmpty()) {
            return UniReturnUtil.fail("服务编号不能为空");
        }
        String serviceId = serviceIdOpt.get();
//        todo
        return UniReturnUtil.success(null);
    }

    public Object processByAlgorithm(String type, String targetSource, String className, String event, String expression,List<String> filterColumns, Map<String, Object> filterLines, Object... args) throws Exception {

        switch (type) {
//            java反射
            case "1" -> {
                if (Objects.nonNull(targetSource)) {
                    File file = new File(System.getProperty("user.dir").concat(File.separator).concat("plugins").concat(File.separator).concat(targetSource));
                    if (!file.exists()) {
                        return "外部文件加载不存在：".concat(System.getProperty("user.dir")).concat(File.separator).concat("plugins").concat(File.separator).concat(targetSource);
                    }
                    Method addURLMethod = null;
                    boolean accessible = false;
                    try {
                        addURLMethod = URLClassLoader.class.getDeclaredMethod("addURL", URL.class);
                        URLClassLoader classLoader = (URLClassLoader) ClassLoader.getSystemClassLoader();
                        accessible = addURLMethod.canAccess(null); //获取方法的访问权限以便恢复:判读是否可以访问: true  代表可以访问
                        if (!accessible) { //判读是否可以访问，如果是false，那就必须setAccessible(true)
                            addURLMethod.setAccessible(true);//实际上是为了提效
                        }
                        addURLMethod.invoke(classLoader, file.toURI().toURL());//加载到系统中
                    } finally {
                        if (addURLMethod != null && !accessible) {
                            addURLMethod.setAccessible(false);//恢复原访问权限
                        }
                    }
                }

                Class<?> classExample = Class.forName(className); //获取类实例
                Method javaMethod = null;
                Object classInstance = classExample.getConstructor().newInstance();//类实例接口 无参数构造
                for (Method currentMethod : classExample.getMethods()) {//循环所有方法
                    String methodName = currentMethod.getName();
                    if (methodName.equals(expression)) {
                        javaMethod = currentMethod;
                        break;
                    }
                }
                if (Objects.isNull(javaMethod)) {
                    throw new MethodNotFoundException(expression + "： 没找到");
                }
                return javaMethod.invoke(classInstance, args);
            }
//            JS表达式
            case "2" -> {
                return ScriptEngineUtil.eval(expression, args);
            }
//            数据库
            case "3" -> {
                String eventStr = expression.trim().substring(0, 6);
                if (Objects.equals(eventStr, "insert") || Objects.equals(eventStr, "update") || Objects.equals(eventStr, "delete") || Objects.equals(eventStr, "select")) {
                    switch (event) {
                        case "1", "2", "3" -> {
                            List<Object[]> argsList = Arrays.stream(args).map(it -> ((Object[]) it)).toList();
                            return DataBase.updateBatch(targetSource, expression, argsList);

                        }
                        case "0" -> {
                            List<Object[]> argsList = Arrays.stream(args).map(it -> ((Object[]) it)).toList();
                            return DataBase.query(targetSource, expression, argsList);
                        }
                        default -> {
                            throw new RuntimeException("event 不能为空");
                        }
                    }
                } else {
                    switch (event) {
                        case "6" -> {

                            Map<String, List<Map<String, Object>>> insertOrUpdate = splitDBData(targetSource, expression, Arrays.stream(args).map(it -> ((Map<String, Object>) it)).toList());
                            List<Map<String, Object>> insertList = insertOrUpdate.get("1");
                            Object o = processByAlgorithm(type, targetSource, className, "1", expression, filterColumns,filterLines,insertList.toArray());
                            List<Map<String, Object>> updateList = insertOrUpdate.get("2");
                            Object o2 = processByAlgorithm(type, targetSource, className, "2", expression,filterColumns,filterLines, updateList.toArray());
                            return null;
                        }
                        case "0" -> {
                            expression = """
                                    select * from %s where 《whereStr》
                                    """.formatted(expression);
                            List<Map<String, Object>> result = DataBase.query(targetSource, expression, Arrays.stream(args).map(it -> ((Map<String, Object>) it)).toList(), filterColumns, filterLines);

                        }
                        case "1" -> {
                            List<Map<String,Object>> objects = Arrays.stream(args).map(it -> (Map<String,Object>)((Map<String, Object>) it).getOrDefault("value", it)).toList();

                            Map<String,Object> o = objects.get(0);
                            List<String> names = o.keySet().stream().filter(it->{
                                if(Objects.isNull(filterColumns) || filterColumns.isEmpty()) return true;
                                return filterColumns.contains(it);
                            }).toList();
                            String sql = "insert into %s(%s) values(%s)".formatted(expression, String.join(",", names), names.stream().map(it->"?").collect(Collectors.joining(",")));

                            return DataBase.updateBatch(targetSource, sql, objects.stream().map(value -> names.stream().map(value::get).toArray()).toList());
                        }
                        case "2" -> {

                        }
                        case "3" -> {

                        }
                        default -> {

                            throw new RuntimeException("event 不能为空");
                        }
                    }
                }
            }
        }
        return null;
    }

    public Map<String, List<Map<String, Object>>> splitDBData(String connectionStr, String tableName, List<Map<String, Object>> argsList) throws Exception {

        Map<String, List<Map<String, Object>>> result = new HashMap<>() {{
            put("1", new ArrayList<>());
            put("2", new ArrayList<>());
        }};
        for (Map<String, Object> args : argsList) {

            if (exist(connectionStr, tableName, args)) {
                result.get("2").add(args);
            } else {
                result.get("1").add(args);
            }
        }

        return result;
    }

    public boolean exist(String connectionStr, String tableName, Map<String, Object> args) throws Exception {

        List<Map<String, Object>> mapList = DataBase.query(connectionStr, "select count(1) as existscount from %s where 《whereStr》 ".formatted(tableName), List.of(args), null, null);

        return !mapList.get(0).get("existscount").equals("0");
    }

}
